classdef truss2dMinCompliance_0 < truss2d 
    % This class minimizes the compliance a 2d truss system
    properties(GetAccess = 'public', SetAccess = 'private')
        myInitialArea;
        myInitialVolume;
        myInitialCompliance;
        myFinalArea;
        myFinalCompliance;
        myFinalVolume;
    end
    methods
        function obj = truss2dMinCompliance_0(xy,connectivity)
            obj = obj@truss2d(xy,connectivity);
        end
        function J = complianceObjective(obj,Area)
            obj = obj.assignA(Area);
            obj = obj.assemble();
            obj = obj.solve();
            J = obj.mySol'*obj.myForceExternal(:);
        end  
        function obj = assignYieldStress(obj,~,~)
            % dummy function
        end
        function obj = initialize(obj)
            obj.myInitialArea = obj.myArea;
            obj.myInitialVolume = sum(obj.myArea.*obj.myL);
            obj = obj.assemble();
            obj = obj.solve();
            obj.myInitialCompliance =  obj.getCompliance(); 
        end
        function obj = optimize(obj)
            obj = obj.initialize();
            LB = 1e-12*ones(1,obj.myNumTrussBars); % small non-zero areas
            AEq = [];
            BEq = [];
            AinEq = obj.myL;
            BinEq = obj.myInitialVolume;  
            [Aoptimal,~,~,Output] = fmincon(@obj.complianceObjective,obj.myInitialArea, ...
                   AinEq,BinEq,AEq,BEq,LB);
            obj = obj.assignA(Aoptimal);
            obj = obj.assemble();
            obj = obj.solve();
            obj.myFinalArea = Aoptimal;
            obj.myFinalVolume = sum(obj.myArea.*obj.myL);
            obj.myFinalCompliance =  obj.getCompliance();    
            disp(Output)
        end
    end
end

