classdef truss2dMinVolumeComplianceConstraint < truss2d 
    % compliance consrained volume minimization
    properties(GetAccess = 'public', SetAccess = 'private')
        myInitialVolume;
        myInitialArea;
        myInitialCompliance;
        myFinalArea;
        myFinalCompliance;
        myFinalVolume;
    end
    methods
        function obj = truss2dMinVolumeComplianceConstraint(xy,connectivity)
            obj = obj@truss2d(xy,connectivity);
        end
        function volRelative = volumeObjective(obj,x)
            Area = x.*obj.myInitialArea;
            obj = obj.assignA(Area);
            vol = sum(obj.myArea.*obj.myL);
            volRelative = vol/obj.myInitialVolume;
        end 
        function [cineq,ceq] = complianceConstraint(obj,x)
            Area = x.*obj.myInitialArea;
            obj = obj.assignA(Area);
            obj = obj.assemble();
            obj = obj.solve();
            J = obj.getCompliance();
            cineq = [];
            ceq = J/obj.myInitialCompliance - 1;
        end
        function obj = assignYieldStress(obj,~,~)
            % dummy function
        end
        function obj = initialize(obj)
            obj.myInitialArea = obj.myArea;
            obj.myInitialVolume = sum(obj.myArea.*obj.myL);
            obj = obj.assemble();
            obj = obj.solve();
            obj.myInitialCompliance =  obj.getCompliance(); 
        end
        function obj = optimize(obj)
            obj = obj.initialize();
            x0 = ones(1,obj.myNumTrussBars); % unitless quantities      
            LB = 1e-8*ones(1,obj.myNumTrussBars); % small non-zero values
            opt = optimoptions('fmincon','PlotFcns',@optimplotfval, 'MaxFunEvals',25000);%
            [xMin,~,~,~]  = fmincon(@obj.volumeObjective,x0, ...
                   [],[],[],[],LB,[],@obj.complianceConstraint,opt);
            obj = obj.assignA(xMin.*obj.myInitialArea);
            obj = obj.assemble();
            obj = obj.solve();
            obj.myFinalArea = obj.myArea;
            obj.myFinalVolume = sum(obj.myArea.*obj.myL);
            obj.myFinalCompliance =  obj.getCompliance(); 
        end
    end
end

