function [f,g,H] = complexDerivatives(fun,x)
% By Yi Cao at Cranfield University, 02/01/2008
% Modified by Krishnan Suresh to handle .m files
f =feval(fun,x);                       % function value
if nargout == 1
   return;
end
n=numel(x);                         % size of independent
g=zeros(n,1);                       % allocate memory for the gradient matrix
h=sqrt(eps);                        % differentiation step size
h2=h*h;                             % square of step size
if nargout == 3
    H=zeros(n);                     % allocate memory for the Hessian matrix
end
for k=1:n                           % loop for each independent variable
    x1=x;                           % reference point
    x1(k)=x1(k)+h*1i;                % increment in kth independent variable
    if nargout>1
        v=feval(fun,x1);            % function call with a comlex step increment
        g(k)=imag(v)/h;             % the kth gradient
    end
    if nargout == 3
        for l=k:n                       % loop for off diagonal Hessian
            x2=x1;                      % reference with kth increment
            x2(l)=x2(l)+h;              % kth + lth (positive real) increment
            u1=feval(fun,x2);                 % function call with a double increment
            x2(l)=x1(l)-h;              % kth + lth (negative real) increment
            u2=feval(fun,x2);                 % function call with a double increment
            H(k,l)=imag(u1-u2)/h2/2;    % Hessian (central + complex step)
            H(l,k)=H(k,l);              % symmetric
        end
    end
end
