function [g] = finiteDifferenceGradient(f,x0,deltaX,method)
% Compute finite difference gradient of f at x0, with stepLength deltaX 
% method: 1,0,-1 for forward,central, backward difference
N = numel(x0);
if (min(abs(deltaX)) ==0) 
    g = []; return;
end
if (numel(deltaX) == 1)
    deltaX = deltaX*ones(1,N);% same step size for all dimensions
end
g = zeros(N,1); % intitialize memory
f0 = feval(f,x0);
for i = 1:N
    x = x0;
    if (method == 1) % forward difference
        x(i) = x(i) + deltaX(i);
        f1 = feval(f,x);
        g(i) = (f1-f0)/deltaX(i);
    elseif (method == -1) % backward difference
        x(i) = x(i) - deltaX(i);
        f1 = feval(f,x);
        g(i) = (f0-f1)/deltaX(i);
    else   % central difference
        x(i) = x(i) + deltaX(i)/2;
        f2 = feval(f,x);
        x(i) = x(i) - deltaX(i); % same as x0-h/2
        f1 = feval(f,x);
        g(i) = (f2-f1)/deltaX(i);
    end
end
