
clc; % clears the command window
format compact; %reduces print space

% Recommend that you run section by section

%% variables, constants, etc
clc; % clears the command window
a = 2;
b = 2*a-1;

x = pi;
y = sin(x);

%% vectors and matrices
clc; % clears the command window
a = [3 2 4.2]; % creates a vector
b = a'; % transpose
c = a*b; % dot product, result in scalar
C  = b*a; % results in a matrix

A = [2 -1 0; -1 2 -1; 0 -1 2];

d = A*b; % matrix-vector product
x = 0:0.1:10; % creates a vector


%% for loop
clc; % clears the command window
tic
x = 0:0.000001:pi;
clear y; % remove all instances of y
for i = 1:numel(x)
    y(i) = sin(x(i))*x(i); % y = sin(x)*x
end
toc

%% for loop with memory allocation ... better
clc; % clears the command window
tic
x = 0:0.000001:pi;
clear y; % remove all instances of y
y = zeros(size(x)); % preallocate memory
for i = numel(x)
    y(i) = sin(x(i))*x(i); % y = sin(x)*x
end
toc
%% for loop with vectorization ... best
clc; % clears the command window
tic
x = 0:0.000001:pi;
clear y
y = sin(x).*x;
toc

%% if-else with display
clc; % clears the command window
for i=1:10
    if i <= 4
        disp([i, i^2])
    elseif i <=7
        disp([i,i^2+1])
    else
        disp([i,i^2+2])
    end
end

%% while with string concatenation
clc; % clears the command window
s = ''; % initialization
while length(s) < 25
    s  = [s '-'];%concatenation
end
disp(s)

%% Linear Algebra
clc; % clears the command window
A = [2 -1; -1 2];
b = [1; 1];
x = A\b; % solves Ax = b ... more efficient than inv(A)*b
[eigVectors,eigValues ] = eig(A); % Solving A*x = (lambda)*x
A = rand(50,50);
x = rand(50,1);
b = A*x;
xComputed = A\b;
errVector = x-xComputed;
err = norm(x-xComputed);
disp(['Error for a random matrix solve is ' num2str(err)])

%% 2-D plots
clc; % clears the command window
x = 0:pi/50:2*pi;
y = sin(x);
z = cos(x);
plot(x,y,'gx'); hold on;
plot(x,z,'r:');
xlabel('x')
ylabel('sin(x) & cos(x)')
legend('sin(x)','cos(x)')

%% 3-D plotting 
clc; % clears the command window
[x,y] = meshgrid(0:.1:2, 3:.1:5);
f = (x-1).^2 + (y-4).^4;
surf(x,y,f);
xlabel('x')
ylabel('y')

%% contour
figure;
[c, h] = contour(x,y,f,20); 
clabel(c, h);
axis equal

%% Symbolic calculations
clc; % clears the command window
syms x real
f = x*sin(x^2);
dfdx = diff(f,x);

g = x*x + 2;
h = int(g); % indefinite integration
h0 = int(g,0,1); % definite integration

%% Statistical calculations
clc; % clears the command window
x = [0.501 0.496 0.510 0.508 0.493 0.507 0.491 0.493 0.492 0.513];
mean(x)
std(x)
min(x)
max(x)
sum(x)

x = rand(1,1000); % uniform
mean(x)
histogram(x)
figure;
y = randn(1,1000); % normal distribution
histogram(y);


%% Complex numbers
clc; % clears the command window
a = 2 + 3*sqrt(-1);
b = 4*a;

%% Simple anonymous functions
clc; % clears the command window
clear x;
f = @(x) (1-x.*exp(-x.^2)); % vectorized, anonymous function
a = f(2);

x = -4:0.01:4;
y = f(x);
plot(x,y,'Linewidth',2.0)
xlabel('x');
ylabel('1-x*exp(-x^2)');

