clc; % clears the command window
format compact; %reduces print space
% Recommend that you run section by section

%% single variable non-linear equation
clc; % clears the command window
fsolve(@(x)fsolveTestFunction1(x),pi/2)

%% non-linear equation arising from optimization problem
clc; % clears the command window
% g1 is the first derivative of f1 = 1- x*exp(-x^2)
fsolve(@(x)g1(x),1)

%% solving 2 non-linear equations in 2 variables
clc; % clears the command window
fsolve(@(xVec)fsolveTestFunction2(xVec),[1 0.5])

%% recursive sum
clc
S = recursiveSum(10)

%% subMatrices
clc
% Example to illustrate sub-matrix indexing
A = rand(3,3) % create a random (3,3) matrix
% Create a new matrix M where the first row and first column are removed
M = A(2:3,2:3)
% Create a new matrix M where the second row and second column are removed
M = A([1 3],[1 3])
% Create a new matrix M where the second row and third column are removed
M = A([1 3],[1 2])

%% modules
clc
test1D = testFunctions1D;
test1D.f1(0.5)

%% Polynomial class
clc
p = PolynomialClass([1 -1 0 3.2]);
q = PolynomialClass([0 2 3]);
p.evaluate(0.5);
p

r = p + q
r.evaluate(1)

%% Polynomial class inheritance
clc
p = PolynomialClassRoots([1 -1 0 3.2]);
p.findRoots()


%% Uniform Sampling
clc
[xMin,fMin] = fminSampling(@(x)cos(x),0,2*pi,100)
[xMin,fMin] = fminSampling(@(x)cos(x),0,2*pi,1000)

test1D = testFunctions1D;

[xMin,fMin] = fminSampling(@(x)test1D.f1(x),-5,5,1000)


%% UI
clc
% uitable example
% create a cell; this allows adding text as entries
% If you only want to display numbers, then use Data = zeros(4,6);
Data = cell(4,6);% 4 experiments, 6 data per expt.

% observe the curly bracket (for cells)
Data(1,:) = {0.33 1e-10 0 1e2  1000.06 'Converged'};
Data(2,:) = {0.1 1e-10 88 1e-2  999 'Failed'};
Data(3,:) = {-0.44 1e-10 1.334 1e2  14.56 'Converged'};
Data(4,:) = {-0.43 0.001 0 1e6  155 'Converged'};
colnames = {'Col-1', 'Col-2', 'Col-3', 'Col-4','Col-5','Comment'};
rowNames = {'Problem-a','Problem-b','Problem-c','Problem-d'};
% we make the table editable so that the column seperators can be dragged
t = uitable(figure, 'Data', Data, 'ColumnName', colnames,'ColumnEditable',true, 'RowName', rowNames,'FontSize',10,...
    'Position',[20 20 600 200]);
