function [xStationary,fValue,flag,output] = fStationaryNR(fun,x0,opt)
% Finds the stationary values of a function via Newton Raphson
% Assume that fun(x) returns function value, gradient and hessian
if (nargin == 2), opt = setDefaultOptimizationOptions(); end % default parameters
xStationary = x0(:);
iter = 0;
flag = 0;
output.message = '#FunEvals exceeded'; % default
debug = false;
if (debug)
    fprintf('-------------------------------------------------\n');
    fprintf('#     f(xi)     norm(g(xi))  \n');
    fprintf('-------------------------------------------------\n');
end
[fValue,g,H] = fun(xStationary);
nFuncCalls = 1;
while(iter < opt.MaxFunEvals)
    if (debug)
        fprintf('%d    %.4e     %.4e \n', iter, fValue, norm(g));
    end
    if (cond(H) > 1e12)
        output.message = 'Hessian could not be inverted';
        flag = 0;
        break;
    end
    if (norm(g) < opt.TolGrad)
        output.message = 'grad Tol met';
        flag = 1;
        break;
    end
    xNew = xStationary - H\g(:);% inverse
    err = norm(xStationary-xNew);
    if (err < opt.TolX)
        output.message = 'X Tol met';
        flag = 1;
        break;
    end
    xStationary = xNew;
    [fValueNew,g,H] = fun(xStationary); 
    nFuncCalls = nFuncCalls + 1;
    % Here, it is OK to test against opt.TolFun
    if (abs(fValueNew - fValue) < opt.TolFun)
        output.message = 'Func Tol met';
        flag = 1;
        break;
     end
    fValue = fValueNew;
    iter = iter + 1;
end
output.iterations = iter;
output.funcCount = nFuncCalls;
