function [xMin,fMin,flag,output] = fminPowell(f,x0,opt)
% Powell's method using line-search
if (nargin == 2), opt = setDefaultOptimizationOptions(); end % default parameters
output.message = '#FunEvals exceeded'; % default
opt = optimset(opt,'display','off'); % suppress annoying messages
x0 = x0(:); % force into column vector
flag = 0;pass = 1;nFunEvals = 0;
N = length(x0); % dimension of the problem
xCurrent = x0;
D = eye(N,N); %% we need to keep track of the direction

while (nFunEvals < opt.MaxFunEvals) % for each pass
    xStartOfPass = xCurrent;
    fStartOfPass = feval(f,xStartOfPass);
    nFunEvals = nFunEvals+1;
    for ls = 1:N % N line search for each pass
        d = D(:,ls); % the direction of the current pass
        [xNew,fNew,flag,funcCount] = lineSearchQuadFit(f,xCurrent,d,opt); 
        xCurrent = xNew;
    end
    xEndOfPass = xCurrent;
    fEndOfPass = feval(f,xEndOfPass);
    nFunEvals = nFunEvals+funcCount;
    pass = pass+1;
    if ( (norm(xEndOfPass -xStartOfPass)<opt.TolX))
       output.message = 'X Tol met'; % default
       break; % terminate
    end
    if ( (abs(fEndOfPass-fStartOfPass) < opt.TolFun))
       output.message = 'Fun Tol met'; % default
       break; % terminate
    end
    % update directions
    D(:,1:end-1) = D(:,2:end);
    D(:,end) = (xEndOfPass-xStartOfPass);
    % One line search along new direction
    [xCurrent,fNew,flag,funcCount] = lineSearchQuadFit(f,xCurrent,D(:,end),opt);
    nFunEvals = nFunEvals+funcCount;
    if (rank(D) < N),D = eye(N,N); end%reset directions
end
xMin = xNew;
fMin = fNew; 
output.funcCount = nFunEvals;
output.D = D;
