function [xMin,fMin,flag,funcCount] = lineSearchQuadFit(f,x0,d,opt)
% Approximate line search of a N-dimensional function using quadratic interpolation
if (nargin == 3)
    opt = setDefaultOptimizationOptions();
end
x0 = x0(:); d = d(:);
d = d/norm(d); % ensure proper scaling
% 
alpha0 = 0;
deltaAlpha = max(1e-2,(opt.TolX)*100);
fa = f1D(f,x0,d,alpha0);
fb = f1D(f,x0,d,alpha0+deltaAlpha);
fc = f1D(f,x0,d,alpha0-deltaAlpha);
a = alpha0; b = alpha0+deltaAlpha; c = alpha0-deltaAlpha;
denom = fa*(b-c) + fb*(c-a) + fc*(a-b);
numer = fa*(b^2-c^2) + fb*(c^2-a^2) + fc*(a^2-b^2);
funcCount = 3;
if (abs(denom) < eps)
    flag = -1;
    xMin = x0;
    fMin = fa;
    disp('Line search failed');
    return;
end
alphaMin = 0.5*numer/denom;
xMin = x0+alphaMin*d;   
fMin = f(xMin);
funcCount = funcCount + 1;
flag = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fValue] = f1D(f,x,d,alpha)
fValue = feval(f,x+alpha*d);% Evaluate f at p+alpha*d
