format long;
clc;
close all
% Comparing fminunc vs. global optimizaton methods on 
% noisy 1D functions 
test1D = testFunctions1D;
nVariables = 1;
timeLimit = 0.05; % seconds
noiseLevel = 0.01;
f = @(x)test1D.f7Noisy(x,noiseLevel);

xSamples = -1:0.01:2;
fSamples= f(xSamples);
plot(xSamples,fSamples);


x0 = 0.5;
problem = createOptimProblem('fmincon','objective',f,'x0',x0);
for method =0:4
    disp('----------------------');
     if (method == 0)
        [xMin,fMin,~,output] = fminunc(f,x0);
        disp(['fminunc: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    elseif (method == 1)
        ms = MultiStart('MaxTime',timeLimit);
        [xMin,fMin,~,output] = run(ms,problem,50);
        disp(['MultiStart: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    elseif (method == 2)
        gs = GlobalSearch('MaxTime',timeLimit);
        [xMin,fMin,~,output] = run(gs,problem);
        disp(['GlobalSearch: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    elseif (method == 3)
        optGA = optimoptions('ga','MaxTime',timeLimit);
        [xMin,fMin,~,output] = ga(f,nVariables,[],[],[],[],[],[],[],optGA);
        disp(['GeneticAlgorithm: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    else
        optSA = optimoptions('simulannealbnd','MaxTime',timeLimit,'MaxFunEvals',1e6);
        [xMin,fMin,~,output] = simulannealbnd(@(x)f(x),0,[],[],optSA);
        disp(['SimulatedAnnealing: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    end
end
