format long;
% Comparing fminunc vs. global optimizaton methods on 
% smooth ND functions with multiple local minima


testND = testFunctionsND;
nVariables = 2;
timeLimit = 5; % seconds
f = @(x)testND.f13(x);

[x,y] = meshgrid(-10:0.25:10,-10:0.25:10);
fVal = sin(x).*exp((1-cos(y)).^2)+cos(y).*exp((1-sin(x)).^ 2)+(x-y).^2; % bird function
surf(x,y,fVal);

figure;
[c, h] = contour(x,y,fVal,20); 
clabel(c, h);
axis equal


x0 = [0 0];
problem = createOptimProblem('fmincon','objective',f,'x0',x0);
for method = 0:4
    disp('----------------------');
    if (method == 0)
        [xMin,fMin,~,output] = fminunc(f,x0);
        disp(['fminunc: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    elseif (method == 1)
        ms = MultiStart('MaxTime',timeLimit);
        [xMin,fMin,~,output] = run(ms,problem,50);
        disp(['MultiStart: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    elseif (method == 2)
        gs = GlobalSearch('MaxTime',timeLimit);
        [xMin,fMin,~,output] = run(gs,problem);
        disp(['GlobalSearch: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    elseif (method == 3)
        optGA = optimoptions('ga','MaxTime',timeLimit);
        [xMin,fMin,~,output] = ga(f,nVariables,[],[],[],[],[],[],[],optGA);
        disp(['GeneticAlgorithm: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    else
        optSA = optimoptions('simulannealbnd','MaxTime',timeLimit,'MaxFunEvals',1e6);
        [xMin,fMin,~,output] = simulannealbnd(@(x)f(x),x0,[],[],optSA);
        disp(['SimulatedAnnealing: xMin = ' num2str(xMin) '; fMin = ' num2str(fMin)]);
    end
end
